@@ -183,7 +183,7 @@ class ContinuousGenerator(keras.utils.Sequence):
     feature_kwargs : dict or list of dicts
         Set of options to pass to the feature when resolving
     ndim : int
-        Number of dimensions of each batch (including the batch dimension).
+        Number of dimensions of each batch (excluding the batch dimension).
     """
 
     def __init__(
@@ -197,7 +197,7 @@ class ContinuousGenerator(keras.utils.Sequence):
         shuffle_batch=True,
         feature_kwargs={},
         verbose=1,
-        ndim=4,
+        ndim=3,
     ):
 
         if min_data_size is None:
@@ -313,18 +313,22 @@ class ContinuousGenerator(keras.utils.Sequence):
             new_image = self._get(self.feature, self.feature_kwargs)
 
             if self.label_function:
-                new_label = self.label_function(new_image)
-                if isinstance(new_label, (np.ndarray, float, int)):
-                    new_label = (new_label,)
+                new_label = Image(self.label_function(new_image))
 
             if self.batch_function:
-                new_image = self.batch_function(new_image)
-            if len(self.data) >= self.max_data_size:
-                self.data[index % self.max_data_size] = (new_image,) + tuple(new_label)
-            else:
-                self.data.append((new_image,) + tuple(new_label))
+                new_image = Image(self.batch_function(new_image))
 
-            index += 1
+            if new_image.ndim < self.ndim:
+                new_image = [new_image]
+                new_label = [new_label]
+
+            for new_image_i, new_label_i in zip(new_image, new_label):
+                if len(self.data) >= self.max_data_size:
+                    self.data[index % self.max_data_size] = (new_image_i, new_label_i)
+                else:
+                    self.data.append((new_image_i, new_label_i))
+
+                index += 1
 
     def _get(self, features: Feature or List[Feature], feature_kwargs) -> Image:
         # Updates and resolves a feature or list of features.
